use anyhow::Result;
use axum::{
    body::Bytes, extract::{
        rejection::JsonRejection, Json, State
    }, response::ErrorResponse, Router
};
use hyper::StatusCode;

use crate::{message::Message, player::Player};
use crate::db::AppState;
use crate::message::MessageKind;

pub struct App {
    pub db: AppState,
    pub router: Router<AppState>,
}

impl App {
    pub fn new() -> Self {
        let db = AppState::new();    
        let router: Router<AppState> = Router::new();

        Self {
            db,
            router,
        }
    }
    
    pub async fn echo(body: Bytes) -> Bytes {
        body
    }

    pub async fn create_player(
        State(db): State<AppState>,
        json_body: Result<Json<Message>, JsonRejection>,
    ) -> Result<axum::Json<Message>, StatusCode>
    {
        match json_body {
            Ok(Json(message)) => {
                println!("{:?}", message);
                let player: Player = serde_json::from_str(&message.content).expect("niente json");
                {
                    let mut players = db.players
                        .lock().expect("ciao");

                    players.insert(
                        player.get_name(),
                        Bytes::copy_from_slice(player.encode().expect("failed encoding player").as_ref())
                    );
                }

                println!("{:?}", db.players);
                Ok(Json(Message::new("server", MessageKind::CreatePlayer, "user created").expect("failed creating message")))
            }
            Err(_) => Err(StatusCode::UNPROCESSABLE_ENTITY)
        }
    }
}
